@@ -1,212 +1,206 @@
 # Copyright (c) OpenMMLab. All rights reserved.
 import warnings
+from collections import OrderedDict
+from typing import Dict, List, Optional, Tuple, Union
 
 import torch
 import torch.nn as nn
-from mmcv.cnn import ConvModule, kaiming_init
-from mmcv.runner import _load_checkpoint, load_checkpoint
-from mmcv.utils import print_log
+from mmcv.cnn import ConvModule
+from mmengine.logging import MMLogger, print_log
+from mmengine.model import BaseModule
+from mmengine.model.weight_init import kaiming_init
+from mmengine.runner.checkpoint import _load_checkpoint, load_checkpoint
 
-from ...utils import get_root_logger
-from ..builder import BACKBONES
+from mmaction.registry import MODELS
 from .resnet3d import ResNet3d
 
-try:
-    from mmdet.models import BACKBONES as MMDET_BACKBONES
-    mmdet_imported = True
-except (ImportError, ModuleNotFoundError):
-    mmdet_imported = False
+
+class DeConvModule(BaseModule):
+    """A deconv module that bundles deconv/norm/activation layers.
+
+    Args:
+        in_channels (int): Number of channels in the input feature map.
+        out_channels (int): Number of channels produced by the convolution.
+        kernel_size (int | tuple[int]): Size of the convolving kernel.
+        stride (int | tuple[int]): Stride of the convolution.
+        padding (int | tuple[int]): Zero-padding added to both sides of
+            the input.
+        bias (bool): Whether to add a learnable bias to the output.
+            Defaults to False.
+        with_bn (bool): Whether to add a BN layer. Defaults to True.
+        with_relu (bool): Whether to add a ReLU layer. Defaults to True.
+    """
+
+    def __init__(self,
+                 in_channels: int,
+                 out_channels: int,
+                 kernel_size: int,
+                 stride: Union[int, Tuple[int]] = (1, 1, 1),
+                 padding: Union[int, Tuple[int]] = 0,
+                 bias: bool = False,
+                 with_bn: bool = True,
+                 with_relu: bool = True) -> None:
+        super().__init__()
+        self.in_channels = in_channels
+        self.out_channels = out_channels
+        self.kernel_size = kernel_size
+        self.stride = stride
+        self.padding = padding
+        self.bias = bias
+        self.with_bn = with_bn
+        self.with_relu = with_relu
+
+        self.conv = nn.ConvTranspose3d(
+            in_channels,
+            out_channels,
+            kernel_size,
+            stride=stride,
+            padding=padding,
+            bias=bias)
+        self.bn = nn.BatchNorm3d(out_channels)
+        self.relu = nn.ReLU()
+
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        """Defines the computation performed at every call."""
+        # x should be a 5-d tensor
+        assert len(x.shape) == 5
+        N, C, T, H, W = x.shape
+        out_shape = (N, self.out_channels, self.stride[0] * T,
+                     self.stride[1] * H, self.stride[2] * W)
+        x = self.conv(x, output_size=out_shape)
+        if self.with_bn:
+            x = self.bn(x)
+        if self.with_relu:
+            x = self.relu(x)
+        return x
 
 
 class ResNet3dPathway(ResNet3d):
     """A pathway of Slowfast based on ResNet3d.
 
     Args:
-        *args (arguments): Arguments same as :class:``ResNet3d``.
         lateral (bool): Determines whether to enable the lateral connection
-            from another pathway. Default: False.
+            from another pathway. Defaults to False.
+        lateral_inv (bool): Whether to use deconv to upscale the time
+            dimension of features from another pathway. Defaults to False.
+        lateral_norm (bool): Determines whether to enable the lateral norm
+            in lateral layers. Defaults to False.
         speed_ratio (int): Speed ratio indicating the ratio between time
             dimension of the fast and slow pathway, corresponding to the
-            ``alpha`` in the paper. Default: 8.
+            ``alpha`` in the paper. Defaults to 8.
         channel_ratio (int): Reduce the channel number of fast pathway
             by ``channel_ratio``, corresponding to ``beta`` in the paper.
-            Default: 8.
+            Defaults to 8.
         fusion_kernel (int): The kernel size of lateral fusion.
-            Default: 5.
-        **kwargs (keyword arguments): Keywords arguments for ResNet3d.
+            Defaults to 5.
+        lateral_infl (int): The ratio of the inflated channels.
+            Defaults to 2.
+        lateral_activate (list[int]): Flags for activating the lateral
+            connection. Defaults to ``[1, 1, 1, 1]``.
     """
 
     def __init__(self,
-                 *args,
-                 lateral=False,
-                 lateral_norm=False,
-                 speed_ratio=8,
-                 channel_ratio=8,
-                 fusion_kernel=5,
-                 **kwargs):
+                 lateral: bool = False,
+                 lateral_inv: bool = False,
+                 lateral_norm: bool = False,
+                 speed_ratio: int = 8,
+                 channel_ratio: int = 8,
+                 fusion_kernel: int = 5,
+                 lateral_infl: int = 2,
+                 lateral_activate: List[int] = [1, 1, 1, 1],
+                 **kwargs) -> None:
         self.lateral = lateral
+        self.lateral_inv = lateral_inv
         self.lateral_norm = lateral_norm
         self.speed_ratio = speed_ratio
         self.channel_ratio = channel_ratio
         self.fusion_kernel = fusion_kernel
-        super().__init__(*args, **kwargs)
+        self.lateral_infl = lateral_infl
+        self.lateral_activate = lateral_activate
+        self._calculate_lateral_inplanes(kwargs)
+
+        super().__init__(**kwargs)
         self.inplanes = self.base_channels
-        if self.lateral:
-            self.conv1_lateral = ConvModule(
-                self.inplanes // self.channel_ratio,
-                # https://arxiv.org/abs/1812.03982, the
-                # third type of lateral connection has out_channel:
-                # 2 * \beta * C
-                self.inplanes * 2 // self.channel_ratio,
-                kernel_size=(fusion_kernel, 1, 1),
-                stride=(self.speed_ratio, 1, 1),
-                padding=((fusion_kernel - 1) // 2, 0, 0),
-                bias=False,
-                conv_cfg=self.conv_cfg,
-                norm_cfg=self.norm_cfg if self.lateral_norm else None,
-                act_cfg=self.act_cfg if self.lateral_norm else None)
+        if self.lateral and self.lateral_activate[0] == 1:
+            if self.lateral_inv:
+                self.conv1_lateral = DeConvModule(
+                    self.inplanes * self.channel_ratio,
+                    self.inplanes * self.channel_ratio // lateral_infl,
+                    kernel_size=(fusion_kernel, 1, 1),
+                    stride=(self.speed_ratio, 1, 1),
+                    padding=((fusion_kernel - 1) // 2, 0, 0),
+                    with_bn=True,
+                    with_relu=True)
+            else:
+                self.conv1_lateral = ConvModule(
+                    self.inplanes // self.channel_ratio,
+                    self.inplanes * lateral_infl // self.channel_ratio,
+                    kernel_size=(fusion_kernel, 1, 1),
+                    stride=(self.speed_ratio, 1, 1),
+                    padding=((fusion_kernel - 1) // 2, 0, 0),
+                    bias=False,
+                    conv_cfg=self.conv_cfg,
+                    norm_cfg=self.norm_cfg if self.lateral_norm else None,
+                    act_cfg=self.act_cfg if self.lateral_norm else None)
 
         self.lateral_connections = []
         for i in range(len(self.stage_blocks)):
             planes = self.base_channels * 2**i
             self.inplanes = planes * self.block.expansion
 
-            if lateral and i != self.num_stages - 1:
+            if lateral and i != self.num_stages - 1 \
+                    and self.lateral_activate[i + 1]:
                 # no lateral connection needed in final stage
                 lateral_name = f'layer{(i + 1)}_lateral'
-                setattr(
-                    self, lateral_name,
-                    ConvModule(
+                if self.lateral_inv:
+                    conv_module = DeConvModule(
+                        self.inplanes * self.channel_ratio,
+                        self.inplanes * self.channel_ratio // lateral_infl,
+                        kernel_size=(fusion_kernel, 1, 1),
+                        stride=(self.speed_ratio, 1, 1),
+                        padding=((fusion_kernel - 1) // 2, 0, 0),
+                        bias=False,
+                        with_bn=True,
+                        with_relu=True)
+                else:
+                    conv_module = ConvModule(
                         self.inplanes // self.channel_ratio,
-                        self.inplanes * 2 // self.channel_ratio,
+                        self.inplanes * lateral_infl // self.channel_ratio,
                         kernel_size=(fusion_kernel, 1, 1),
                         stride=(self.speed_ratio, 1, 1),
                         padding=((fusion_kernel - 1) // 2, 0, 0),
                         bias=False,
                         conv_cfg=self.conv_cfg,
                         norm_cfg=self.norm_cfg if self.lateral_norm else None,
-                        act_cfg=self.act_cfg if self.lateral_norm else None))
+                        act_cfg=self.act_cfg if self.lateral_norm else None)
+                setattr(self, lateral_name, conv_module)
                 self.lateral_connections.append(lateral_name)
 
-    def make_res_layer(self,
-                       block,
-                       inplanes,
-                       planes,
-                       blocks,
-                       spatial_stride=1,
-                       temporal_stride=1,
-                       dilation=1,
-                       style='pytorch',
-                       inflate=1,
-                       inflate_style='3x1x1',
-                       non_local=0,
-                       non_local_cfg=dict(),
-                       conv_cfg=None,
-                       norm_cfg=None,
-                       act_cfg=None,
-                       with_cp=False):
-        """Build residual layer for Slowfast.
+    def _calculate_lateral_inplanes(self, kwargs):
+        """Calculate inplanes for lateral connection."""
+        depth = kwargs.get('depth', 50)
+        expansion = 1 if depth < 50 else 4
+        base_channels = kwargs.get('base_channels', 64)
+        lateral_inplanes = []
+        for i in range(kwargs.get('num_stages', 4)):
+            if expansion % 2 == 0:
+                planes = base_channels * (2 ** i) * \
+                         ((expansion // 2) ** (i > 0))
+            else:
+                planes = base_channels * (2**i) // (2**(i > 0))
+            if self.lateral and self.lateral_activate[i]:
+                if self.lateral_inv:
+                    lateral_inplane = planes * \
+                                      self.channel_ratio // self.lateral_infl
+                else:
+                    lateral_inplane = planes * \
+                                      self.lateral_infl // self.channel_ratio
+            else:
+                lateral_inplane = 0
+            lateral_inplanes.append(lateral_inplane)
+        self.lateral_inplanes = lateral_inplanes
 
-        Args:
-            block (nn.Module): Residual module to be built.
-            inplanes (int): Number of channels for the input
-                feature in each block.
-            planes (int): Number of channels for the output
-                feature in each block.
-            blocks (int): Number of residual blocks.
-            spatial_stride (int | Sequence[int]): Spatial strides
-                in residual and conv layers. Default: 1.
-            temporal_stride (int | Sequence[int]): Temporal strides in
-                residual and conv layers. Default: 1.
-            dilation (int): Spacing between kernel elements. Default: 1.
-            style (str): ``pytorch`` or ``caffe``. If set to ``pytorch``,
-                the stride-two layer is the 3x3 conv layer,
-                otherwise the stride-two layer is the first 1x1 conv layer.
-                Default: ``pytorch``.
-            inflate (int | Sequence[int]): Determine whether to inflate
-                for each block. Default: 1.
-            inflate_style (str): ``3x1x1`` or ``3x3x3``. which determines
-                the kernel sizes and padding strides for conv1 and
-                conv2 in each block. Default: ``3x1x1``.
-            non_local (int | Sequence[int]): Determine whether to apply
-                non-local module in the corresponding block of each stages.
-                Default: 0.
-            non_local_cfg (dict): Config for non-local module.
-                Default: ``dict()``.
-            conv_cfg (dict | None): Config for conv layers. Default: None.
-            norm_cfg (dict | None): Config for norm layers. Default: None.
-            act_cfg (dict | None): Config for activate layers. Default: None.
-            with_cp (bool): Use checkpoint or not. Using checkpoint will save
-                some memory while slowing down the training speed.
-                Default: False.
-
-        Returns:
-            nn.Module: A residual layer for the given config.
-        """
-        inflate = inflate if not isinstance(inflate,
-                                            int) else (inflate, ) * blocks
-        non_local = non_local if not isinstance(
-            non_local, int) else (non_local, ) * blocks
-        assert len(inflate) == blocks and len(non_local) == blocks
-        if self.lateral:
-            lateral_inplanes = inplanes * 2 // self.channel_ratio
-        else:
-            lateral_inplanes = 0
-        if (spatial_stride != 1
-                or (inplanes + lateral_inplanes) != planes * block.expansion):
-            downsample = ConvModule(
-                inplanes + lateral_inplanes,
-                planes * block.expansion,
-                kernel_size=1,
-                stride=(temporal_stride, spatial_stride, spatial_stride),
-                bias=False,
-                conv_cfg=conv_cfg,
-                norm_cfg=norm_cfg,
-                act_cfg=None)
-        else:
-            downsample = None
-
-        layers = []
-        layers.append(
-            block(
-                inplanes + lateral_inplanes,
-                planes,
-                spatial_stride,
-                temporal_stride,
-                dilation,
-                downsample,
-                style=style,
-                inflate=(inflate[0] == 1),
-                inflate_style=inflate_style,
-                non_local=(non_local[0] == 1),
-                non_local_cfg=non_local_cfg,
-                conv_cfg=conv_cfg,
-                norm_cfg=norm_cfg,
-                act_cfg=act_cfg,
-                with_cp=with_cp))
-        inplanes = planes * block.expansion
-
-        for i in range(1, blocks):
-            layers.append(
-                block(
-                    inplanes,
-                    planes,
-                    1,
-                    1,
-                    dilation,
-                    style=style,
-                    inflate=(inflate[i] == 1),
-                    inflate_style=inflate_style,
-                    non_local=(non_local[i] == 1),
-                    non_local_cfg=non_local_cfg,
-                    conv_cfg=conv_cfg,
-                    norm_cfg=norm_cfg,
-                    act_cfg=act_cfg,
-                    with_cp=with_cp))
-
-        return nn.Sequential(*layers)
-
-    def inflate_weights(self, logger):
+    def inflate_weights(self, logger: MMLogger) -> None:
         """Inflate the resnet2d parameters to resnet3d pathway.
 
         The differences between resnet3d and resnet2d mainly lie in an extra
@@ -216,11 +210,11 @@ class ResNet3dPathway(ResNet3d):
         not be inflated from 2d weights.
 
         Args:
-            logger (logging.Logger): The logger used to print
+            logger (MMLogger): The logger used to print
                 debugging information.
         """
 
-        state_dict_r2d = _load_checkpoint(self.pretrained)
+        state_dict_r2d = _load_checkpoint(self.pretrained, map_location='cpu')
         if 'state_dict' in state_dict_r2d:
             state_dict_r2d = state_dict_r2d['state_dict']
 
@@ -263,8 +257,9 @@ class ResNet3dPathway(ResNet3d):
             logger.info(f'These parameters in the 2d checkpoint are not loaded'
                         f': {remaining_names}')
 
-    def _inflate_conv_params(self, conv3d, state_dict_2d, module_name_2d,
-                             inflated_param_names):
+    def _inflate_conv_params(self, conv3d: nn.Module,
+                             state_dict_2d: OrderedDict, module_name_2d: str,
+                             inflated_param_names: List[str]) -> None:
         """Inflate a conv module from 2d to 3d.
 
         The differences of conv modules betweene 2d and 3d in Pathway
@@ -312,7 +307,7 @@ class ResNet3dPathway(ResNet3d):
             conv3d.bias.data.copy_(state_dict_2d[bias_2d_name])
             inflated_param_names.append(bias_2d_name)
 
-    def _freeze_stages(self):
+    def _freeze_stages(self) -> None:
         """Prevent all the parameters from being optimized before
         `self.frozen_stages`."""
         if self.frozen_stages >= 0:
@@ -334,7 +329,7 @@ class ResNet3dPathway(ResNet3d):
                 for param in conv_lateral.parameters():
                     param.requires_grad = False
 
-    def init_weights(self, pretrained=None):
+    def init_weights(self, pretrained: Optional[str] = None) -> None:
         """Initiate the parameters either from existing checkpoint or from
         scratch."""
         if pretrained:
@@ -355,12 +350,12 @@ pathway_cfg = {
 }
 
 
-def build_pathway(cfg, *args, **kwargs):
+def build_pathway(cfg: Dict, *args, **kwargs) -> nn.Module:
     """Build pathway.
 
     Args:
-        cfg (None or dict): cfg should contain:
-            - type (str): identify conv layer type.
+        cfg (dict): cfg should contain:
+            - type (str): identify backbone type.
 
     Returns:
         nn.Module: Created pathway.
@@ -379,8 +374,8 @@ def build_pathway(cfg, *args, **kwargs):
     return pathway
 
 
-@BACKBONES.register_module()
-class ResNet3dSlowFast(nn.Module):
+@MODELS.register_module()
+class ResNet3dSlowFast(BaseModule):
     """Slowfast backbone.
 
     This module is proposed in `SlowFast Networks for Video Recognition
@@ -393,53 +388,40 @@ class ResNet3dSlowFast(nn.Module):
             multipling the ``interval`` in ``SampleFrames`` in the
             pipeline with ``resample_rate``, equivalent to the :math:`\\tau`
             in the paper, i.e. it processes only one out of
-            ``resample_rate * interval`` frames. Default: 8.
+            ``resample_rate * interval`` frames. Defaults to 8.
         speed_ratio (int): Speed ratio indicating the ratio between time
             dimension of the fast and slow pathway, corresponding to the
-            :math:`\\alpha` in the paper. Default: 8.
+            :math:`\\alpha` in the paper. Defaults to 8.
         channel_ratio (int): Reduce the channel number of fast pathway
             by ``channel_ratio``, corresponding to :math:`\\beta` in the paper.
-            Default: 8.
-        slow_pathway (dict): Configuration of slow branch, should contain
-            necessary arguments for building the specific type of pathway
-            and:
-            type (str): type of backbone the pathway bases on.
-            lateral (bool): determine whether to build lateral connection
-            for the pathway.Default:
-
-            .. code-block:: Python
-
-                dict(type='ResNetPathway',
-                lateral=True, depth=50, pretrained=None,
-                conv1_kernel=(1, 7, 7), dilations=(1, 1, 1, 1),
-                conv1_stride_t=1, pool1_stride_t=1, inflate=(0, 0, 1, 1))
-
-        fast_pathway (dict): Configuration of fast branch, similar to
-            `slow_pathway`. Default:
-
-            .. code-block:: Python
-
-                dict(type='ResNetPathway',
-                lateral=False, depth=50, pretrained=None, base_channels=8,
-                conv1_kernel=(5, 7, 7), conv1_stride_t=1, pool1_stride_t=1)
+            Defaults to 8.
+        slow_pathway (dict): Configuration of slow branch. Defaults to
+            ``dict(type='resnet3d', depth=50, pretrained=None, lateral=True,
+            conv1_kernel=(1, 7, 7), conv1_stride_t=1, pool1_stride_t=1,
+            inflate=(0, 0, 1, 1))``.
+        fast_pathway (dict): Configuration of fast branch. Defaults to
+            ``dict(type='resnet3d', depth=50, pretrained=None, lateral=False,
+            base_channels=8, conv1_kernel=(5, 7, 7), conv1_stride_t=1,
+            pool1_stride_t=1)``.
+        init_cfg (dict or list[dict], optional): Initialization config dict.
+            Defaults to None.
     """
 
     def __init__(self,
-                 pretrained,
-                 resample_rate=8,
-                 speed_ratio=8,
-                 channel_ratio=8,
-                 slow_pathway=dict(
+                 pretrained: Optional[str] = None,
+                 resample_rate: int = 8,
+                 speed_ratio: int = 8,
+                 channel_ratio: int = 8,
+                 slow_pathway: Dict = dict(
                      type='resnet3d',
                      depth=50,
                      pretrained=None,
                      lateral=True,
                      conv1_kernel=(1, 7, 7),
-                     dilations=(1, 1, 1, 1),
                      conv1_stride_t=1,
                      pool1_stride_t=1,
                      inflate=(0, 0, 1, 1)),
-                 fast_pathway=dict(
+                 fast_pathway: Dict = dict(
                      type='resnet3d',
                      depth=50,
                      pretrained=None,
@@ -447,8 +429,9 @@ class ResNet3dSlowFast(nn.Module):
                      base_channels=8,
                      conv1_kernel=(5, 7, 7),
                      conv1_stride_t=1,
-                     pool1_stride_t=1)):
-        super().__init__()
+                     pool1_stride_t=1),
+                 init_cfg: Optional[Union[Dict, List[Dict]]] = None) -> None:
+        super().__init__(init_cfg=init_cfg)
         self.pretrained = pretrained
         self.resample_rate = resample_rate
         self.speed_ratio = speed_ratio
@@ -461,14 +444,14 @@ class ResNet3dSlowFast(nn.Module):
         self.slow_path = build_pathway(slow_pathway)
         self.fast_path = build_pathway(fast_pathway)
 
-    def init_weights(self, pretrained=None):
+    def init_weights(self, pretrained: Optional[str] = None) -> None:
         """Initiate the parameters either from existing checkpoint or from
         scratch."""
         if pretrained:
             self.pretrained = pretrained
 
         if isinstance(self.pretrained, str):
-            logger = get_root_logger()
+            logger = MMLogger.get_current_instance()
             msg = f'load model from: {self.pretrained}'
             print_log(msg, logger=logger)
             # Directly load 3D model.
@@ -480,15 +463,15 @@ class ResNet3dSlowFast(nn.Module):
         else:
             raise TypeError('pretrained must be a str or None')
 
-    def forward(self, x):
+    def forward(self, x: torch.Tensor) -> tuple:
         """Defines the computation performed at every call.
 
         Args:
             x (torch.Tensor): The input data.
 
         Returns:
-            tuple[torch.Tensor]: The feature of the input samples extracted
-                by the backbone.
+            tuple[torch.Tensor]: The feature of the input samples
+                extracted by the backbone.
         """
         x_slow = nn.functional.interpolate(
             x,
@@ -525,7 +508,3 @@ class ResNet3dSlowFast(nn.Module):
         out = (x_slow, x_fast)
 
         return out
-
-
-if mmdet_imported:
-    MMDET_BACKBONES.register_module()(ResNet3dSlowFast)
