@@ -1,14 +1,13 @@
 # Copyright (c) OpenMMLab. All rights reserved.
 from .conv2plus1d import Conv2plus1d
 from .conv_audio import ConvAudio
-from .lfb import LFB
 from .sub_batchnorm3d import SubBatchNorm3D
 from .tam import TAM
 from .transformer import (DividedSpatialAttentionWithNorm,
                           DividedTemporalAttentionWithNorm, FFNWithNorm)
 
 __all__ = [
-    'Conv2plus1d', 'ConvAudio', 'LFB', 'TAM',
-    'DividedSpatialAttentionWithNorm', 'DividedTemporalAttentionWithNorm',
-    'FFNWithNorm', 'SubBatchNorm3D'
+    'Conv2plus1d', 'TAM', 'DividedSpatialAttentionWithNorm',
+    'DividedTemporalAttentionWithNorm', 'FFNWithNorm', 'SubBatchNorm3D',
+    'ConvAudio'
 ]
