@@ -3,23 +3,21 @@ import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+from mmengine.model import BaseModel
+from mmengine.model.weight_init import constant_init, kaiming_init
 
-from ...localization import temporal_iop
-from ..builder import LOCALIZERS, build_loss
-from .base import BaseTAPGenerator
-from .utils import post_processing
+from mmaction.registry import MODELS
+from .utils import post_processing, temporal_iop
 
 
-@LOCALIZERS.register_module()
-class TEM(BaseTAPGenerator):
+@MODELS.register_module()
+class TEM(BaseModel):
     """Temporal Evaluation Model for Boundary Sensitive Network.
 
     Please refer `BSN: Boundary Sensitive Network for Temporal Action
     Proposal Generation <http://arxiv.org/abs/1806.02964>`_.
-
     Code reference
     https://github.com/wzmsltw/BSN-boundary-sensitive-network
-
     Args:
         tem_feat_dim (int): Feature dimension.
         tem_hidden_dim (int): Hidden layer dimension.
@@ -53,7 +51,7 @@ class TEM(BaseTAPGenerator):
         self.c_hidden = tem_hidden_dim
         self.match_threshold = tem_match_threshold
         self.output_dim = output_dim
-        self.loss_cls = build_loss(loss_cls)
+        self.loss_cls = MODELS.build(loss_cls)
         self.loss_weight = loss_weight
         self.conv1_ratio = conv1_ratio
         self.conv2_ratio = conv2_ratio
@@ -81,15 +79,23 @@ class TEM(BaseTAPGenerator):
             padding=0)
         self.anchors_tmins, self.anchors_tmaxs = self._temporal_anchors()
 
+    def init_weights(self) -> None:
+        """Initiate the parameters either from existing checkpoint or from
+        scratch."""
+        for m in self.modules():
+            if isinstance(m, nn.Conv2d):
+                kaiming_init(m)
+            elif isinstance(m, nn.BatchNorm2d):
+                constant_init(m, 1)
+
     def _temporal_anchors(self, tmin_offset=0., tmax_offset=1.):
         """Generate temporal anchors.
 
         Args:
             tmin_offset (int): Offset for the minimum value of temporal anchor.
                 Default: 0.
-            tmax_offset (int): Offset for the maximun value of temporal anchor.
+            tmax_offset (int): Offset for the maximum value of temporal anchor.
                 Default: 1.
-
         Returns:
             tuple[Sequence[float]]: The minimum and maximum values of temporal
                 anchors.
@@ -108,7 +114,6 @@ class TEM(BaseTAPGenerator):
 
         Args:
             x (torch.Tensor): The input data.
-
         Returns:
             torch.Tensor: The output of the module.
         """
@@ -117,40 +122,58 @@ class TEM(BaseTAPGenerator):
         x = torch.sigmoid(self.conv3_ratio * self.conv3(x))
         return x
 
-    def forward_train(self, raw_feature, label_action, label_start, label_end):
-        """Define the computation performed at every call when training."""
-        tem_output = self._forward(raw_feature)
+    def loss(self, batch_inputs, batch_data_samples, **kwargs):
+        """Calculate losses from a batch of inputs and data samples.
+
+        Args:
+            batch_inputs (Tensor): Raw Inputs of the recognizer.
+                These should usually be mean centered and std scaled.
+            batch_data_samples (List[:obj:`ActionDataSample`]): The batch
+                data samples. It usually includes information such
+                as ``gt_labels``.
+
+        Returns:
+            dict: A dictionary of loss components.
+        """
+        tem_output = self._forward(batch_inputs)
+
         score_action = tem_output[:, 0, :]
         score_start = tem_output[:, 1, :]
         score_end = tem_output[:, 2, :]
 
+        gt_bbox = [
+            sample.gt_instances['gt_bbox'] for sample in batch_data_samples
+        ]
+        label_action, label_start, label_end = self.generate_labels(gt_bbox)
+        device = batch_inputs.device
+        label_action = label_action.to(device)
+        label_start = label_start.to(device)
+        label_end = label_end.to(device)
+
         loss_action = self.loss_cls(score_action, label_action,
                                     self.match_threshold)
-        loss_start_small = self.loss_cls(score_start, label_start,
-                                         self.match_threshold)
-        loss_end_small = self.loss_cls(score_end, label_end,
-                                       self.match_threshold)
+        loss_start = self.loss_cls(score_start, label_start,
+                                   self.match_threshold)
+        loss_end = self.loss_cls(score_end, label_end, self.match_threshold)
+
         loss_dict = {
             'loss_action': loss_action * self.loss_weight,
-            'loss_start': loss_start_small,
-            'loss_end': loss_end_small
+            'loss_start': loss_start,
+            'loss_end': loss_end
         }
 
         return loss_dict
 
-    def forward_test(self, raw_feature, video_meta):
+    def predict(self, batch_inputs, batch_data_samples, **kwargs):
         """Define the computation performed at every call when testing."""
-        tem_output = self._forward(raw_feature).cpu().numpy()
+        tem_output = self._forward(batch_inputs).cpu().numpy()
         batch_action = tem_output[:, 0, :]
         batch_start = tem_output[:, 1, :]
         batch_end = tem_output[:, 2, :]
 
-        video_meta_list = [dict(x) for x in video_meta]
-
         video_results = []
-
         for batch_idx, _ in enumerate(batch_action):
-            video_name = video_meta_list[batch_idx]['video_name']
+            video_name = batch_data_samples[batch_idx].metainfo['video_name']
             video_action = batch_action[batch_idx]
             video_start = batch_start[batch_idx]
             video_end = batch_end[batch_idx]
@@ -162,6 +185,7 @@ class TEM(BaseTAPGenerator):
 
     def generate_labels(self, gt_bbox):
         """Generate training labels."""
+        # TODO: do this without numpy
         match_score_action_list = []
         match_score_start_list = []
         match_score_end_list = []
@@ -206,35 +230,57 @@ class TEM(BaseTAPGenerator):
         return (match_score_action_list, match_score_start_list,
                 match_score_end_list)
 
-    def forward(self,
-                raw_feature,
-                gt_bbox=None,
-                video_meta=None,
-                return_loss=True):
-        """Define the computation performed at every call."""
-        if return_loss:
-            label_action, label_start, label_end = (
-                self.generate_labels(gt_bbox))
-            device = raw_feature.device
-            label_action = label_action.to(device)
-            label_start = label_start.to(device)
-            label_end = label_end.to(device)
-            return self.forward_train(raw_feature, label_action, label_start,
-                                      label_end)
+    def forward(self, inputs, data_samples, mode, **kwargs):
+        """The unified entry for a forward process in both training and test.
 
-        return self.forward_test(raw_feature, video_meta)
+        The method should accept three modes:
+
+        - ``tensor``: Forward the whole network and return tensor or tuple of
+        tensor without any post-processing, same as a common nn.Module.
+        - ``predict``: Forward and return the predictions, which are fully
+        processed to a list of :obj:`ActionDataSample`.
+        - ``loss``: Forward and return a dict of losses according to the given
+        inputs and data samples.
+
+        Note that this method doesn't handle neither back propagation nor
+        optimizer updating, which are done in the :meth:`train_step`.
+
+        Args:
+            inputs (Tensor): The input tensor with shape
+                (N, C, ...) in general.
+            data_samples (List[:obj:`ActionDataSample`], optional): The
+                annotation data of every samples. Defaults to None.
+            mode (str): Return what kind of value. Defaults to ``tensor``.
+
+        Returns:
+            The return type depends on ``mode``.
+
+            - If ``mode="tensor"``, return a tensor or a tuple of tensor.
+            - If ``mode="predict"``, return a list of ``ActionDataSample``.
+            - If ``mode="loss"``, return a dict of tensor.
+        """
+        if type(inputs) is not torch.Tensor:
+            inputs = torch.stack(inputs)
+
+        if mode == 'tensor':
+            return self._forward(inputs, **kwargs)
+        if mode == 'predict':
+            return self.predict(inputs, data_samples, **kwargs)
+        elif mode == 'loss':
+            return self.loss(inputs, data_samples, **kwargs)
+        else:
+            raise RuntimeError(f'Invalid mode "{mode}". '
+                               'Only supports loss, predict and tensor mode')
 
 
-@LOCALIZERS.register_module()
-class PEM(BaseTAPGenerator):
+@MODELS.register_module()
+class PEM(BaseModel):
     """Proposals Evaluation Model for Boundary Sensitive Network.
 
     Please refer `BSN: Boundary Sensitive Network for Temporal Action
     Proposal Generation <http://arxiv.org/abs/1806.02964>`_.
-
     Code reference
     https://github.com/wzmsltw/BSN-boundary-sensitive-network
-
     Args:
         pem_feat_dim (int): Feature dimension.
         pem_hidden_dim (int): Hidden layer dimension.
@@ -255,20 +301,20 @@ class PEM(BaseTAPGenerator):
     """
 
     def __init__(self,
-                 pem_feat_dim,
-                 pem_hidden_dim,
-                 pem_u_ratio_m,
-                 pem_u_ratio_l,
-                 pem_high_temporal_iou_threshold,
-                 pem_low_temporal_iou_threshold,
-                 soft_nms_alpha,
-                 soft_nms_low_threshold,
-                 soft_nms_high_threshold,
-                 post_process_top_k,
-                 feature_extraction_interval=16,
-                 fc1_ratio=0.1,
-                 fc2_ratio=0.1,
-                 output_dim=1):
+                 pem_feat_dim: int,
+                 pem_hidden_dim: int,
+                 pem_u_ratio_m: float,
+                 pem_u_ratio_l: float,
+                 pem_high_temporal_iou_threshold: float,
+                 pem_low_temporal_iou_threshold: float,
+                 soft_nms_alpha: float,
+                 soft_nms_low_threshold: float,
+                 soft_nms_high_threshold: float,
+                 post_process_top_k: int,
+                 feature_extraction_interval: int = 16,
+                 fc1_ratio: float = 0.1,
+                 fc2_ratio: float = 0.1,
+                 output_dim: int = 1):
         super().__init__()
 
         self.feat_dim = pem_feat_dim
@@ -293,26 +339,52 @@ class PEM(BaseTAPGenerator):
             out_features=self.output_dim,
             bias=True)
 
+    def init_weights(self) -> None:
+        """Initiate the parameters either from existing checkpoint or from
+        scratch."""
+        for m in self.modules():
+            if isinstance(m, nn.Conv2d):
+                kaiming_init(m)
+            elif isinstance(m, nn.BatchNorm2d):
+                constant_init(m, 1)
+
     def _forward(self, x):
         """Define the computation performed at every call.
 
         Args:
             x (torch.Tensor): The input data.
-
         Returns:
             torch.Tensor: The output of the module.
         """
-        x = torch.cat(list(x))
         x = F.relu(self.fc1_ratio * self.fc1(x))
         x = torch.sigmoid(self.fc2_ratio * self.fc2(x))
         return x
 
-    def forward_train(self, bsp_feature, reference_temporal_iou):
-        """Define the computation performed at every call when training."""
+    def loss(self, batch_inputs, batch_data_samples, **kwargs):
+        """Calculate losses from a batch of inputs and data samples.
+
+        Args:
+            batch_inputs (Tensor): Raw Inputs of the recognizer.
+                These should usually be mean centered and std scaled.
+            batch_data_samples (List[:obj:`ActionDataSample`]): The batch
+                data samples. It usually includes information such
+                as ``gt_labels``.
+
+        Returns:
+            dict: A dictionary of loss components.
+        """
+        device = self.fc1.weight.device
+
+        bsp_feature = torch.cat([
+            sample.gt_instances['bsp_feature'] for sample in batch_data_samples
+        ]).to(device)
+
+        reference_temporal_iou = torch.cat([
+            sample.gt_instances['reference_temporal_iou']
+            for sample in batch_data_samples
+        ]).to(device)
+
         pem_output = self._forward(bsp_feature)
-        reference_temporal_iou = torch.cat(list(reference_temporal_iou))
-        device = pem_output.device
-        reference_temporal_iou = reference_temporal_iou.to(device)
 
         anchors_temporal_iou = pem_output.view(-1)
         u_hmask = (reference_temporal_iou >
@@ -350,21 +422,35 @@ class PEM(BaseTAPGenerator):
 
         return loss_dict
 
-    def forward_test(self, bsp_feature, tmin, tmax, tmin_score, tmax_score,
-                     video_meta):
+    def _parse(self, gt_instances, key):
+        out = torch.cat([gt[key] for gt in gt_instances])
+        out = out.view(-1).cpu().numpy().reshape(-1, 1)
+        return out
+
+    def predict(self, batch_inputs, batch_data_samples, **kwargs):
         """Define the computation performed at every call when testing."""
-        pem_output = self._forward(bsp_feature).view(-1).cpu().numpy().reshape(
-            -1, 1)
+        device = self.fc1.weight.device
+
+        bsp_feature = torch.cat([
+            sample.gt_instances['bsp_feature'] for sample in batch_data_samples
+        ]).to(device)
+
+        pem_output = self._forward(bsp_feature).view(-1).cpu().numpy()
+        pem_output = pem_output.reshape(-1, 1)
+
+        gt_instances = [sample.gt_instances for sample in batch_data_samples]
+
+        tmin = self._parse(gt_instances, 'tmin')
+        tmax = self._parse(gt_instances, 'tmax')
+        tmin_score = self._parse(gt_instances, 'tmin_score')
+        tmax_score = self._parse(gt_instances, 'tmax_score')
 
-        tmin = tmin.view(-1).cpu().numpy().reshape(-1, 1)
-        tmax = tmax.view(-1).cpu().numpy().reshape(-1, 1)
-        tmin_score = tmin_score.view(-1).cpu().numpy().reshape(-1, 1)
-        tmax_score = tmax_score.view(-1).cpu().numpy().reshape(-1, 1)
         score = np.array(pem_output * tmin_score * tmax_score).reshape(-1, 1)
         result = np.concatenate(
             (tmin, tmax, tmin_score, tmax_score, pem_output, score), axis=1)
         result = result.reshape(-1, 6)
-        video_info = dict(video_meta[0])
+
+        video_info = batch_data_samples[0].metainfo
         proposal_list = post_processing(result, video_info,
                                         self.soft_nms_alpha,
                                         self.soft_nms_low_threshold,
@@ -378,18 +464,42 @@ class PEM(BaseTAPGenerator):
         ]
         return output
 
-    def forward(self,
-                bsp_feature,
-                reference_temporal_iou=None,
-                tmin=None,
-                tmax=None,
-                tmin_score=None,
-                tmax_score=None,
-                video_meta=None,
-                return_loss=True):
-        """Define the computation performed at every call."""
-        if return_loss:
-            return self.forward_train(bsp_feature, reference_temporal_iou)
+    def forward(self, inputs, data_samples, mode, **kwargs):
+        """The unified entry for a forward process in both training and test.
 
-        return self.forward_test(bsp_feature, tmin, tmax, tmin_score,
-                                 tmax_score, video_meta)
+        The method should accept three modes:
+
+        - ``tensor``: Forward the whole network and return tensor or tuple of
+        tensor without any post-processing, same as a common nn.Module.
+        - ``predict``: Forward and return the predictions, which are fully
+        processed to a list of :obj:`ActionDataSample`.
+        - ``loss``: Forward and return a dict of losses according to the given
+        inputs and data samples.
+
+        Note that this method doesn't handle neither back propagation nor
+        optimizer updating, which are done in the :meth:`train_step`.
+
+        Args:
+            batch_inputs (Tensor): The input tensor with shape
+                (N, C, ...) in general.
+            batch_data_samples (List[:obj:`ActionDataSample`], optional): The
+                annotation data of every samples. Defaults to None.
+            mode (str): Return what kind of value. Defaults to ``tensor``.
+
+        Returns:
+            The return type depends on ``mode``.
+
+            - If ``mode="tensor"``, return a tensor or a tuple of tensor.
+            - If ``mode="predict"``, return a list of ``ActionDataSample``.
+            - If ``mode="loss"``, return a dict of tensor.
+        """
+        inputs = torch.stack(inputs)
+        if mode == 'tensor':
+            return self._forward(inputs, **kwargs)
+        if mode == 'predict':
+            return self.predict(inputs, data_samples, **kwargs)
+        elif mode == 'loss':
+            return self.loss(inputs, data_samples, **kwargs)
+        else:
+            raise RuntimeError(f'Invalid mode "{mode}". '
+                               'Only supports loss, predict and tensor mode')
